package com.snowm.scanner.helpers

import android.app.PendingIntent
import android.content.Context
import android.content.Intent
import com.google.android.gms.location.Geofence
import com.google.android.gms.location.GeofencingClient
import com.google.android.gms.location.GeofencingRequest
import com.google.android.gms.location.LocationServices
import java.util.ArrayList

class GeofencingHelper(var context: Context) {
    private var geofencingClient: GeofencingClient = LocationServices.getGeofencingClient(context)
    private var geofenceList: ArrayList<Geofence> = ArrayList()
    fun startGeoFencing(identifier: String, geofence: HashMap<String, Any>) {
        geofenceList.add(Geofence.Builder()
                .setRequestId(identifier)
                .setCircularRegion(
                        geofence["latitude"] as Double, geofence["longitude"] as Double, geofence["radius"] as Float
                )
                .setExpirationDuration(Geofence.NEVER_EXPIRE)
                .setTransitionTypes(Geofence.GEOFENCE_TRANSITION_ENTER or Geofence.GEOFENCE_TRANSITION_EXIT)
                .build())
        geofencingClient.addGeofences(getGeofencingRequest(), geofencePendingIntent)
    }

    fun stopGepfencing(identifier: String) {
        val ids = ArrayList<String>(1)
        ids.add(identifier)
        geofencingClient.removeGeofences(ids)
    }

    fun getGeofencingRequest(): GeofencingRequest {
        return GeofencingRequest.Builder().apply {
            setInitialTrigger(GeofencingRequest.INITIAL_TRIGGER_ENTER)
            addGeofences(geofenceList)
        }.build()
    }

    private val geofencePendingIntent: PendingIntent by lazy {
        val intent = Intent()
        intent.setPackage(context.packageName)
        intent.action = "io.snowm.geofence"
        PendingIntent.getBroadcast(context, 0, intent, PendingIntent.FLAG_UPDATE_CURRENT)
    }
}