package com.snowm.scanner.services

import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import android.content.SharedPreferences
import android.location.Location
import android.os.Bundle
import com.google.android.gms.location.*
import com.google.gson.Gson
import com.snowm.scanner.helpers.SnowMBackgroundScanner.Companion.startBackgroundScanning
import com.snowm.scanner.helpers.SnowMBackgroundScanner.Companion.stopBackgroundScanning


open class ScannerBroadcastReceiver : BroadcastReceiver() {
    lateinit var context: Context
    override fun onReceive(context: Context, intent: Intent) {
        this.context = context
        val sharedPreferences: SharedPreferences = context.getSharedPreferences("snowm_scanner", Context.MODE_PRIVATE)
        if (intent.action == "android.intent.action.BOOT_COMPLETED") {
            val scannerDataString = sharedPreferences.getString("backgroundScanner", null)
             if (scannerDataString != null) {
                val bundle: Bundle = Gson().fromJson(scannerDataString, Bundle::class.java) as Bundle
                val geofenceEnabled = bundle.getBoolean("geofenceEnabled")
                if (geofenceEnabled) {
                    val geofence = bundle.getSerializable("geofence") as Map<String, Any>?
                    val centerLatitude = geofence?.get("latitude") as Double
                    val centerLongitude = geofence?.get("longitude") as Double
                    val radius = geofence?.get("longitude") as Int
                    val locationProviderClient = LocationServices.getFusedLocationProviderClient(context)
                    locationProviderClient.lastLocation.addOnCompleteListener {
                        if (!it.isSuccessful) {
                            startBackgroundScanning(context, bundle)
                        } else if (!isWithinGeofence(centerLatitude, centerLongitude, it.result?.latitude!!, it.result?.longitude!!, radius))
                            startBackgroundScanning(context, bundle)
                    }
                } else {
                    startBackgroundScanning(context, bundle)
                }
            }
        } else if (intent.action == "io.snowm.geofence") {
            val geofencingEvent = GeofencingEvent.fromIntent(intent)
            if (geofencingEvent.hasError()) {
                val errorMessage: String = GeofenceStatusCodes.getStatusCodeString(geofencingEvent.errorCode)
                return
            }
            geofencingEvent.triggeringGeofences.forEach {
                if (geofencingEvent.geofenceTransition == Geofence.GEOFENCE_TRANSITION_ENTER) {
                    didEnterGeofence()
                    if (it.requestId == "backgroundGeofence") {
                        val scannerDataString = sharedPreferences.getString("backgroundScanner", null)
                        if (scannerDataString != null) {
                            val bundle: Bundle = Gson().fromJson(scannerDataString, Bundle::class.java) as Bundle
                            startBackgroundScanning(context, bundle)
                        }
                    }
                } else if (geofencingEvent.geofenceTransition == Geofence.GEOFENCE_TRANSITION_EXIT) {
                    didExitGeofence()
                    if (it.requestId == "backgroundGeofence")
                        stopBackgroundScanning(context)
                }

            }
        }
    }


    fun isWithinGeofence(centerLatitude: Double, centerLongitude: Double, testLatitude: Double, testLongitude: Double, radius: Int): Boolean {
        val results = FloatArray(1)
        Location.distanceBetween(centerLatitude, centerLongitude, testLatitude, testLongitude, results)
        val distanceInMeters = results[0]
        return distanceInMeters < radius
    }

    open fun didEnterGeofence() {}

    open fun didExitGeofence() {}

}